@@ -21,6 +21,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
+#include <limits.h>
 #include <unistd.h>
 #include <inttypes.h>
 #include <sys/types.h>
@@ -894,6 +896,65 @@ buildDisplayRequest(char **argv,
     return TRUE;
 }
 
+static void
+setUserAutobrightness(gboolean enable)
+{
+    GVariant *ret = NULL;
+    GError *error = NULL;
+    
+    ret = g_dbus_proxy_call_sync(powerd_proxy,
+            "userAutobrightnessEnable",
+            g_variant_new("(b)", enable),
+            G_DBUS_CALL_FLAGS_NONE,
+            -1, NULL, &error);
+    if (!ret) {
+        cli_warn("userAutobrightnessEnable failed: %s", error->message);
+        g_error_free(error);
+    } else {
+        g_variant_unref(ret);
+    }
+}
+
+static gboolean
+getBrightnessParams(int *min, int *max, int *dflt, gboolean *ab_supported)
+{
+    GVariant *ret = NULL;
+    GError *error = NULL;
+
+    ret = g_dbus_proxy_call_sync(powerd_proxy,
+            "getBrightnessParams",
+            NULL,
+            G_DBUS_CALL_FLAGS_NONE,
+            -1, NULL, &error);
+    if (!ret) {
+        cli_warn("getBrightnessParams failed: %s", error->message);
+        g_error_free(error);
+        return FALSE;
+    }
+
+    g_variant_get(ret, "((iiib))", min, max, dflt, ab_supported);
+    g_variant_unref(ret);
+    return TRUE;
+}
+
+static void setUserBrightness(int brightness)
+{
+    GVariant *ret = NULL;
+    GError *error = NULL;
+    
+    ret = g_dbus_proxy_call_sync(powerd_proxy,
+            "setUserBrightness",
+            g_variant_new("(i)", brightness),
+            G_DBUS_CALL_FLAGS_NONE,
+            -1, NULL, &error);
+    if (!ret) {
+        cli_warn("setUserBrightness failed: %s", error->message);
+        g_error_free(error);
+    } else {
+        g_variant_unref(ret);
+    }
+}
+
 static void
 usage(const char *progname)
 {
@@ -901,25 +962,28 @@ usage(const char *progname)
     printf("active - request the active state, prevent the system from\n"\
            "\tsuspending. This does not modify the screen state.\n");
     printf("active-nc - same as active, but relies on powerd to cleanup\n"\
-            "\tthe request. This is only for testing and will immediately"\
+            "\tthe request. This is only for testing and will immediately\n"\
             "exit, causing the request to be dropped.\n");
-    printf("clear-sys <cookie> - clear a System state request given a cookie\n");
-    printf("clear-disp <cookie> - clear a Display state request given a cookie\n");
-    printf("client-test - test powerd registration / ack API\n");
+    printf("autobrightness <enable|disable> - enable or disable autobrightness.\n");
+    printf("brightness <brightness> - set user screen brightness.\n");
+    printf("brightness-params - get brightness parameters (min, max, etc.)\n");
+    printf("clear-sys <cookie> - clear a System state request given a cookie.\n");
+    printf("clear-disp <cookie> - clear a Display state request given a cookie.\n");
+    printf("client-test - test powerd registration / ack API.\n");
     printf("display <on|dc> [bright] [proximity] [disableab]\n"\
            "\tMake a display state request with the input parameters.\n"\
            "\tThe first argument represents the state of the display:\n"\
-           "\tOn, Off, or Don't Care (dc).\n"\
+           "\tOn (on) or Don't Care (dc),\n"\
            "\tThe final optional arguments respectively:\n"\
            "\t * make the screen bright [bright]\n"\
            "\t * enable the proximity sensor [proximity]\n"\
            "\t * disable autobrightness [disableab]\n");
-    printf("help - display this usage information\n");
-    printf("list - list outstanding requests\n");
-    printf("listen - listen for signals from powerd. This runs a\n"\
-           "\tgmainloop and must be manually killed.\n");
-    printf("stats - print request statistics\n");
-    printf("test - runs tests\n");
+    printf("help\t- display this usage information.\n");
+    printf("list\t- list outstanding requests.\n");
+    printf("listen\t- listen for signals from powerd. This runs a\n"\
+           "\t  gmainloop and must be manually killed.\n");
+    printf("stats\t- print request statistics.\n");
+    printf("test\t- runs tests.\n");
 }
 
 static void
@@ -955,7 +1019,10 @@ int main (int argc, char **argv)
        (strcmp(argv[1],"help")) &&
        (strcmp(argv[1],"clear-disp")) &&
        (strcmp(argv[1],"clear-sys")) &&
-       (strcmp(argv[1],"client-test"))) {
+       (strcmp(argv[1],"client-test")) &&
+       (strcmp(argv[1], "autobrightness")) &&
+       (strcmp(argv[1], "brightness-params")) &&
+       (strcmp(argv[1], "brightness"))) {
         fprintf(stderr,"Invalid option %s\n",argv[1]);
         usage(argv[0]);
         return -1;
@@ -968,8 +1035,7 @@ int main (int argc, char **argv)
 
     myeuid = geteuid();
     if (myeuid != 0) {
-        fprintf(stderr,"You must be root to run %s\n",argv[0]);
-        return -1;
+        fprintf(stderr,"%s: Running as user is not fully supported.\n", argv[0]);
     }
 
     signal(SIGINT, sigint_quit);
@@ -1108,6 +1174,47 @@ int main (int argc, char **argv)
     }
     else if (!strcmp(argv[1], "client-test")) {
         powerd_cli_client_test(powerd_proxy);
+    } else if (!strcmp(argv[1], "autobrightness")) {
+        gboolean enable;
+        if (argc != 3) {
+            usage(argv[0]);
+            exit(-1);
+        }
+        if (!strcmp(argv[2], "enable")) {
+            enable = TRUE;
+        } else if (!strcmp(argv[2], "disable")) {
+            enable = FALSE;
+        } else {
+            fprintf(stderr, "Invalid autobrightness state\n");
+            usage(argv[0]);
+            exit(-1);
+        }
+        setUserAutobrightness(enable);
+    } else if (!strcmp(argv[1], "brightness-params")) {
+        int min, max, dflt, ab;
+        if (getBrightnessParams(&min, &max, &dflt, &ab)) {
+            printf("Minimum Brightness: %d\n"
+                   "Maximum Brightness: %d\n"
+                   "Default Brightness: %d\n"
+                   "Autobrightness:     %ssupported\n",
+                   min, max, dflt, ab ? "" : "not ");
+        }
+    } else if (!strcmp(argv[1], "brightness")) {
+        long brightness;
+        char *endp;
+        if (argc != 3) {
+            usage(argv[0]);
+            exit(-1);
+        }
+        errno = 0;
+        brightness = strtol(argv[2], &endp, 10);
+        if (errno == ERANGE || *endp != '\0' ||
+            brightness < 0 || brightness > INT_MAX) {
+            fprintf(stderr, "Invalid brightness %s\n", argv[2]);
+            usage(argv[0]);
+            exit(-1);
+        }
+        setUserBrightness((int)brightness);
     }
     return 0;
 }
