@@ -86,6 +86,8 @@ static guint state_machine_source_id;
 static gint update_pending;
 static gboolean ack_pending;
 
+static gboolean suspend_active = FALSE;
+
 static const char power_request_wakelock_name[] = "powerd_power_request";
 gint suspend_block_count;
 
@@ -357,6 +359,29 @@ handle_clear_sys_state(PowerdSource *obj, GDBusMethodInvocation *invocation,
     return TRUE;
 }
 
+static int enter_suspend(void)
+{
+    int ret = libsuspend_enter_suspend();
+    if (!ret)
+        suspend_active = TRUE;
+    return ret;
+}
+
+static int exit_suspend(void)
+{
+    int ret = libsuspend_exit_suspend();
+    if (!ret) {
+        suspend_active = FALSE;
+        powerd_display_exit_suspend();
+    }
+    return ret;
+}
+
+gboolean powerd_suspend_active(void)
+{
+    return suspend_active;
+}
+
 /* Must only be called from main loop */
 static enum SysPowerStates max_requested_state(void)
 {
@@ -394,7 +419,7 @@ static enum state_transition_state process_state_dequeue(guint *delay)
 
     if (current_system_state == POWERD_SYS_STATE_SUSPEND) {
         powerd_debug("exiting suspend");
-        ret = libsuspend_exit_suspend();
+        ret = exit_suspend();
         if (ret)
             powerd_warn("Failed to exit suspend: %d", ret);
     }
@@ -448,7 +473,7 @@ static enum state_transition_state process_state_complete(guint *delay)
 
     if (pending_system_state == POWERD_SYS_STATE_SUSPEND && allow_suspend) {
         powerd_debug("entering suspend");
-        ret = libsuspend_enter_suspend();
+        ret = enter_suspend();
         if (ret)
             powerd_warn("Failed to enter suspend: %d", ret);
     }
